--[[-------------------------------------------------------------------------
-- ClosetGnome_Switcher - Swaps your set to match buff names. Can be easily
--   used to enable special outfits for stances and shapeshift forms.
--
--  by PProvost
--
--  Last Modified: $Date: 2006-11-21 12:43:26 -0500 (Tue, 21 Nov 2006) $
--  Revision: $Revision: 17672 $
--
-- TODO:
--  * N/A
--
--  Known Bugs:
--  * N/A
--]]-------------------------------------------------------------------------


--[[ Local Variables and Constants ]]
local MAJOR_VERSION = "0.2"
local MINOR_VERSION = tonumber((string.gsub("$Revision: 17672 $", "^.-(%d+).-$", "%1")))
local currentSetName = nil

--[[ Utility Libraries ]]
local L = AceLibrary("AceLocale-2.2"):new("ClosetGnomeSwitcher")

--[[ Initialize Addon instance ]]
ClosetGnomeSwitcher = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceModuleCore-2.0")
ClosetGnomeSwitcher.version = MAJOR_VERSION .. "." .. MINOR_VERSION
ClosetGnomeSwitcher.date = string.gsub("$Date: 2006-11-21 12:43:26 -0500 (Tue, 21 Nov 2006) $", "^.-(%d%d%d%d%-%d%d%-%d%d).-$", "%1")
ClosetGnomeSwitcher:RegisterDB("ClosetGnomeSwitcherDB", "ClosetGnomeSwitcherDBPC")
ClosetGnomeSwitcher:RegisterDefaults('char', {
		defaultSet = nil,
})
ClosetGnomeSwitcher.sets = {}

ClosetGnomeSwitcher.Opts = {
	name = L["Switcher"],
	desc = L["Tells your closet gnome when to switch outfits for you."],
	type = 'group',
	order = 106,
	args = {
		defaultSet = {
			type = 'text',
			name = L["Default"],
			desc = L["The default set to use when none of the others are applied"],
			usage = '<setName>',
			validate = ClosetGnomeSwitcher.sets,
			get = function() return ClosetGnomeSwitcher.db.char.defaultSet or "None" end,
			set = function(v) ClosetGnomeSwitcher.db.char.defaultSet = v ~= "None" and v or nil end,
		},
		enabled = {
			type = 'toggle',
			name = L["Enabled"],
			desc = L["Enable/disable ClosetGnome_Switcher"],
			get = function() return ClosetGnomeSwitcher:IsActive() end,
			set = function(v) ClosetGnomeSwitcher:ToggleActive(v) end,
		},
	}
}

function ClosetGnomeSwitcher:OnInitialize()
	ClosetGnome.OnMenuRequest.args.switcher = ClosetGnomeSwitcher.Opts
end

function ClosetGnomeSwitcher:OnEnable()
	self:ScheduleRepeatingEvent(self.TimerCallback, 1.0, self) 

	self:RegisterEvent("ClosetGnome_DeleteSet", "UpdateSet")
	self:RegisterEvent("ClosetGnome_AddSet", "UpdateSet")

	if AceEvent:IsFullyInitialized() then
		self:FullyInitialized()
	else
		self:RegisterEvent("AceEvent_FullyInitialized", "FullyInitialized")
	end
end

-- Maintain a sorted list of sets, preceded by a "None" option
function ClosetGnomeSwitcher:FullyInitialized()
	for k, v in pairs(ClosetGnome.db.char.set) do
		table.insert(self.sets, k)
	end
	table.sort(self.sets)
	table.insert(self.sets, 1, "None")
end

function ClosetGnomeSwitcher:UpdateSet(name)
	if ClosetGnome:HasSet(name) then
		table.remove(self.sets, 1)
		table.insert(self.sets, name)
		table.sort(self.sets)
		table.insert(self.sets, 1, "None")
	else
		for k,v in ipairs(self.sets) do
			if v == name then
				table.remove(self.sets, k)
			end
		end
	end
end

function ClosetGnomeSwitcher:TimerCallback()
 	if UnitIsDeadOrGhost("player") == 1 then return end
	local handled = false
	for name, module in self:IterateModules() do
		handled = module:DoSwitch()
		if handled == true then
			break
		end
	end
	if not handled and self.db.char.defaultSet then
		self:WearSet(self.db.char.defaultSet)
	end
end

function ClosetGnomeSwitcher:WearSet(setName)
	if currentSetName ~= setName and ClosetGnome:HasSet(setName) then 
		currentSetName = setName
		ClosetGnome:WearSet(setName) 
	end
end

function ClosetGnomeSwitcher:GetCurrentShapeshiftForm()
    local numForms = GetNumShapeshiftForms()
    for i = 1, numForms do
        local _, name, active = GetShapeshiftFormInfo(i);
        if active == 1 then return name end
    end
    return nil
end
